"use strict";

const IncidentManager = require("../lib/IncidentsManager");


module.exports = {
	metadata: () => ({
		name: "Custom.GetUserIncidents",
		properties: {
			userEmail: { required: true, type: "string" },
			incidentsVariable: {required: true, type: "string"},
			domainName: {required: true, type: "string"},
			hostNamePrefix: {required: true, type: "string"},
			userName: {required: true, type: "string"},
			password: {required: true, type: "string"},
			numberOfIncidentsToFetch: {required: true, type: "int"}
      
		},
		supportedActions: ["error", "found" , "notFound"]
	}),
	invoke: async (context) => {
		
		try {

			// Check if component input variables have valid values
			validateComponentProperties(context);

			// Retrieve component properties.
			const { userEmail,incidentsVariable,domainName,hostNamePrefix, userName, password, numberOfIncidentsToFetch } = context.properties();

			// Create a new Instance of Incident Manager
			const incidentManager = new IncidentManager(hostNamePrefix, domainName, userName, password);

			// Get Incidents from Service Cloud
			const incidents = await incidentManager.getIncidents(userEmail, numberOfIncidentsToFetch);

			// Set extracted incidents into ODA variable
			context.setVariable(incidentsVariable , incidents);

			// Set transition
			context.transition(incidents.length > 0 ? "found" : "notFound");
			
		} catch (error) {
			const errorMessage = error.message;
			// Set Error Transition
			context.transition("error");
			// Set error message variable
			context.setVariable("system.errorAction" , errorMessage);

			// Log Error Message
			context.logger().error(errorMessage);
		}
	}
};



/**
 * Validate component properties.
 * @param {object} context ODA conversation context
 */
function validateComponentProperties(context)
{
	// Get Component properties defined
	const componentProperties = module.exports.metadata().properties;

	// Loop over component properties if any and validate their values
	Object.keys(componentProperties).forEach(propertyName => {
		const property = componentProperties[propertyName];
		const {required, type} = property;
		
		// Get Property values from conversation context for further inspection
		const propertyValue = context.properties()[propertyName];

		// If propertyValue is required and its value is null, then throw an error
		if(required && (!propertyValue || propertyValue === "undefined")){
			throw new Error(`property '${propertyName}' is required.`);
		}

		// If propertyValue is a freeMarker Expression string, then throw an error
		if( type.toLowerCase() === "string" && propertyValue.startsWith("${")){
			throw new Error(`property '${propertyName}' resolved to a FreeMarker Expression string.`);
		}
		else if(type.toLowerCase() === "int" && isNaN(propertyValue)){
			throw new Error(`property '${propertyName}' is not a number.`);
		}

	});

}
