"use strict";


module.exports = {
	metadata: {
		name: "Events.UserInfoHandler",    
		eventHandlerType: "ResolveEntities"
	},
	handlers: { 
		entity: { 
			/**
			 * This method is executed when the Composite Bag is resolved
			 * It copies the Bag Items values (firstName, lastName, email) into (profile.firstName, profile.lastName, email) respectively.
			 * @param {object} event
			 * @param {object} context - entity resolution context, see https://oracle.github.io/bots-node-sdk/EntityResolutionContext.html
			 */
			resolved: async (event, context) => { 

				try {
					// Get resolved bag item values
					const firstName = context.getItemValue("firstName");
					const lastName = context.getItemValue("lastName");
					const email = context.getItemValue("email");

					// Set profile.firstName based on value resolved from CB items
					if(firstName){
						context.setVariable("profile.firstName" , toCamelCase(firstName));
					}

					// Set profile.lastName based on value resolved from CB items
					if(lastName){
						context.setVariable("profile.lastName" , toCamelCase(lastName));
					}

					// Set profile.email based on value resolved from CB items	
					if(email){
						context.setVariable("profile.email" , email.email.toLowerCase());
					}
	
				} catch (error) {

					const errorMessage = error.message;
					// Set error message variable
					context.setVariable("system.errorAction" , errorMessage);

					// Log Error Message
					context.logger().error(errorMessage);
				}
			}
		},
		items: {
			firstName: {
				/**
				 * This method is used to determine if ODA should prompt user to provide a value or not
				 * Only prompt user for his firstName if (profile.firstName) is empty, or if it has invalid values.
				 * If (profile.firstName) has a valid value, then set the firstName bag item value accordingly, and
				 * don't prompt user for input
				 * @param {object} event - event object contains no properties for this handler
				 * @param {object} context - entity resolution context, see https://oracle.github.io/bots-node-sdk/EntityResolutionContext.html
				 * @returns {boolean} should ODA prompt user for input or not for this Bag Item
				 */
				shouldPrompt: async (event, context) => {
					return shouldPromptBagItem("firstName" , context);
				},

			},
			lastName: {
				/**
				 * This method is used to determine if ODA should prompt user to provide a value or not
				 * Only prompt user for his lastName if (profile.lastName) is empty, or if it has invalid values.
				 * If (profile.lastName) has a valid value, then set the lastName bag item value accordingly, and
				 * don't prompt user for input
				 * @param {object} event - event object contains no properties for this handler
				 * @param {object} context - entity resolution context, see https://oracle.github.io/bots-node-sdk/EntityResolutionContext.html
				 * @returns {boolean} should ODA prompt user for input or not for this Bag Item
				 */
				shouldPrompt: async (event, context) => {
					return shouldPromptBagItem("lastName" , context);
				}
			},
			email: {
				/**
				 * This method is used to determine if ODA should prompt user to provide a value or not
				 * Only prompt user for his email if (profile.email) is empty, or if it has invalid values.
				 * If (profile.email) has a valid value, then set the lastName bag item value accordingly, and
				 * don't prompt user for input
				 * @param {object} event - event object contains no properties for this handler
				 * @param {object} context - entity resolution context, see https://oracle.github.io/bots-node-sdk/EntityResolutionContext.html
				 * @returns {boolean} should ODA prompt user for input or not for this Bag Item
				 */
				shouldPrompt: async (event, context) => {
					return shouldPromptBagItem("email" , context);
				}
			}
		}
	}
}; 

/**
 * Convert a text to camel case
 * @param {string} text text to convert
 * @returns {string} text converted to Camel Case
 */
function toCamelCase(text){

	text = text.substr(0,1).toUpperCase() + text.substr(1).toLowerCase();
	return text;
}

/**
 * Check if Bag Item should be prompted for a value. valid Bag Item names are (firstName, lastName, email).
 * The method check if there is a corresponding valid profile value for the bag item name (profile.firstName|lastName|email),
 * If profile variable has a valid value, then the Bag Item is not prompted for value
 * and the bag item value is set to the same corresponding profile variable value.
 * Otherwise, the Bag Item is prompted for a value.
 * 
 * @param {string} bagItemName bag item name (firstName,lastName,email)
 * @param {object} context entity resolution context, see https://oracle.github.io/bots-node-sdk/EntityResolutionContext.html
 * @returns {boolean} should Bag Item be prompted for value
 */
function shouldPromptBagItem(bagItemName, context){
	try {
		// Get the value of (profile.firstName|lastName|email) variable
		const profileValue = context.getVariable(`profile.${bagItemName}`);

		// if (profile.firstName|lastName|email) is invalid (empty|unspecified|or an|invalid email), then ask user to provide corresponding value
		if(!profileValue || profileValue.toLowerCase() == "unspecified" || (bagItemName == "email" ? !profileValue.includes("@") : profileValue.includes("@"))){
			// returning (true) indicates user is prompted to enter a value
			return true;
		}

		// returning (false) indicates user is not prompted to enter a value
		return false;
	} catch (error) {

		const errorMessage = error.message;

		// Set Error Transition
		context.transition("error");

		// Set error message variable
		context.setVariable("system.errorAction" , errorMessage);

		// Log Error Message
		context.logger().error(errorMessage);
	}
}


